@@ -3,6 +3,7 @@ from __future__ import print_function, division, absolute_import
 from abc import ABCMeta, abstractmethod, abstractproperty
 from datetime import timedelta
 import logging
+import weakref
 
 import dask
 from six import with_metaclass
@@ -37,6 +38,12 @@ class Comm(with_metaclass(ABCMeta)):
     depending on the underlying transport's characteristics.
     """
 
+    _instances = weakref.WeakSet()
+
+    def __init__(self):
+        self._instances.add(self)
+        self.name = None
+
     # XXX add set_close_callback()?
 
     @abstractmethod
@@ -116,12 +123,15 @@ class Comm(with_metaclass(ABCMeta)):
         if self.closed():
             return "<closed %s>" % (clsname,)
         else:
-            return ("<%s local=%s remote=%s>"
-                    % (clsname, self.local_address, self.peer_address))
+            return "<%s %s local=%s remote=%s>" % (
+                clsname,
+                self.name or "",
+                self.local_address,
+                self.peer_address,
+            )
 
 
 class Listener(with_metaclass(ABCMeta)):
-
     @abstractmethod
     def start(self):
         """
@@ -158,7 +168,6 @@ class Listener(with_metaclass(ABCMeta)):
 
 
 class Connector(with_metaclass(ABCMeta)):
-
     @abstractmethod
     def connect(self, address, deserialize=True):
         """
@@ -177,8 +186,8 @@ def connect(addr, timeout=None, deserialize=True, connection_args=None):
     retried until the *timeout* is expired.
     """
     if timeout is None:
-        timeout = dask.config.get('distributed.comm.timeouts.connect')
-    timeout = parse_timedelta(timeout, default='seconds')
+        timeout = dask.config.get("distributed.comm.timeouts.connect")
+    timeout = parse_timedelta(timeout, default="seconds")
 
     scheme, loc = parse_address(addr)
     backend = registry.get_backend(scheme)
@@ -190,18 +199,24 @@ def connect(addr, timeout=None, deserialize=True, connection_args=None):
 
     def _raise(error):
         error = error or "connect() didn't finish in time"
-        msg = ("Timed out trying to connect to %r after %s s: %s"
-               % (addr, timeout, error))
+        msg = "Timed out trying to connect to %r after %s s: %s" % (
+            addr,
+            timeout,
+            error,
+        )
         raise IOError(msg)
 
     # This starts a thread
     while True:
         try:
-            future = connector.connect(loc, deserialize=deserialize,
-                                       **(connection_args or {}))
-            comm = yield gen.with_timeout(timedelta(seconds=deadline - time()),
-                                          future,
-                                          quiet_exceptions=EnvironmentError)
+            future = connector.connect(
+                loc, deserialize=deserialize, **(connection_args or {})
+            )
+            comm = yield gen.with_timeout(
+                timedelta(seconds=deadline - time()),
+                future,
+                quiet_exceptions=EnvironmentError,
+            )
         except FatalCommClosedError:
             raise
         except EnvironmentError as e:
@@ -231,13 +246,14 @@ def listen(addr, handle_comm, deserialize=True, connection_args=None):
     try:
         scheme, loc = parse_address(addr, strict=True)
     except ValueError:
-        if connection_args and connection_args.get('ssl_context'):
-            addr = 'tls://' + addr
+        if connection_args and connection_args.get("ssl_context"):
+            addr = "tls://" + addr
         else:
-            addr = 'tcp://' + addr
+            addr = "tcp://" + addr
         scheme, loc = parse_address(addr, strict=True)
 
     backend = registry.get_backend(scheme)
 
-    return backend.get_listener(loc, handle_comm, deserialize,
-                                **(connection_args or {}))
+    return backend.get_listener(
+        loc, handle_comm, deserialize, **(connection_args or {})
+    )
