@@ -11,8 +11,6 @@ try:
 except ImportError:
     ssl = None
 
-from concurrent.futures import ThreadPoolExecutor
-
 import dask
 import tornado
 from tornado import gen, netutil
@@ -21,14 +19,21 @@ from tornado.tcpclient import TCPClient
 from tornado.tcpserver import TCPServer
 
 from ..compatibility import finalize, PY3
-from ..utils import (ensure_bytes, ensure_ip, get_ip, get_ipv6, nbytes,
-                     parse_timedelta, shutting_down)
+from ..threadpoolexecutor import ThreadPoolExecutor
+from ..utils import (
+    ensure_bytes,
+    ensure_ip,
+    get_ip,
+    get_ipv6,
+    nbytes,
+    parse_timedelta,
+    shutting_down,
+)
 
 from .registry import Backend, backends
 from .addressing import parse_host_port, unparse_host_port
 from .core import Comm, Connector, Listener, CommClosedError, FatalCommClosedError
-from .utils import (to_frames, from_frames,
-                    get_tcp_server_address, ensure_concrete_host)
+from .utils import to_frames, from_frames, get_tcp_server_address, ensure_concrete_host
 
 
 logger = logging.getLogger(__name__)
@@ -37,6 +42,7 @@ logger = logging.getLogger(__name__)
 def get_total_physical_memory():
     try:
         import psutil
+
         return psutil.virtual_memory().total / 2
     except ImportError:
         return 2e9
@@ -52,8 +58,8 @@ def set_tcp_timeout(stream):
     if stream.closed():
         return
 
-    timeout = dask.config.get('distributed.comm.timeouts.tcp')
-    timeout = int(parse_timedelta(timeout, default='seconds'))
+    timeout = dask.config.get("distributed.comm.timeouts.tcp")
+    timeout = int(parse_timedelta(timeout, default="seconds"))
 
     sock = stream.socket
 
@@ -69,8 +75,7 @@ def set_tcp_timeout(stream):
 
     try:
         if sys.platform.startswith("win"):
-            logger.debug("Setting TCP keepalive: idle=%d, interval=%d",
-                         idle, interval)
+            logger.debug("Setting TCP keepalive: idle=%d, interval=%d", idle, interval)
             sock.ioctl(socket.SIO_KEEPALIVE_VALS, (1, idle * 1000, interval * 1000))
         else:
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
@@ -87,15 +92,18 @@ def set_tcp_timeout(stream):
                     TCP_KEEPIDLE = None
 
             if TCP_KEEPIDLE is not None:
-                logger.debug("Setting TCP keepalive: nprobes=%d, idle=%d, interval=%d",
-                             nprobes, idle, interval)
+                logger.debug(
+                    "Setting TCP keepalive: nprobes=%d, idle=%d, interval=%d",
+                    nprobes,
+                    idle,
+                    interval,
+                )
                 sock.setsockopt(socket.SOL_TCP, TCP_KEEPCNT, nprobes)
                 sock.setsockopt(socket.SOL_TCP, TCP_KEEPIDLE, idle)
                 sock.setsockopt(socket.SOL_TCP, TCP_KEEPINTVL, interval)
 
         if sys.platform.startswith("linux"):
-            logger.debug("Setting TCP user timeout: %d ms",
-                         timeout * 1000)
+            logger.debug("Setting TCP user timeout: %d ms", timeout * 1000)
             TCP_USER_TIMEOUT = 18  # since Linux 2.6.37
             sock.setsockopt(socket.SOL_TCP, TCP_USER_TIMEOUT, timeout * 1000)
     except EnvironmentError as e:
@@ -124,8 +132,10 @@ def convert_stream_closed_error(obj, exc):
         # The stream was closed because of an underlying OS error
         exc = exc.real_error
         if ssl and isinstance(exc, ssl.SSLError):
-            if 'UNKNOWN_CA' in exc.reason:
-                raise FatalCommClosedError("in %s: %s: %s" % (obj, exc.__class__.__name__, exc))
+            if "UNKNOWN_CA" in exc.reason:
+                raise FatalCommClosedError(
+                    "in %s: %s: %s" % (obj, exc.__class__.__name__, exc)
+                )
         raise CommClosedError("in %s: %s: %s" % (obj, exc.__class__.__name__, exc))
     else:
         raise CommClosedError("in %s: %s" % (obj, exc))
@@ -135,12 +145,14 @@ class TCP(Comm):
     """
     An established communication based on an underlying Tornado IOStream.
     """
+
     _iostream_allows_memoryview = tornado.version_info >= (4, 5)
     # IOStream.read_into() currently proposed in
     # https://github.com/tornadoweb/tornado/pull/2193
     _iostream_has_read_into = hasattr(IOStream, "read_into")
 
     def __init__(self, stream, local_addr, peer_addr, deserialize=True):
+        Comm.__init__(self)
         self._local_addr = local_addr
         self._peer_addr = peer_addr
         self.stream = stream
@@ -180,9 +192,9 @@ class TCP(Comm):
 
         try:
             n_frames = yield stream.read_bytes(8)
-            n_frames = struct.unpack('Q', n_frames)[0]
+            n_frames = struct.unpack("Q", n_frames)[0]
             lengths = yield stream.read_bytes(8 * n_frames)
-            lengths = struct.unpack('Q' * n_frames, lengths)
+            lengths = struct.unpack("Q" * n_frames, lengths)
 
             frames = []
             for length in lengths:
@@ -194,7 +206,7 @@ class TCP(Comm):
                     else:
                         frame = yield stream.read_bytes(length)
                 else:
-                    frame = b''
+                    frame = b""
                 frames.append(frame)
         except StreamClosedError as e:
             self.stream = None
@@ -202,9 +214,9 @@ class TCP(Comm):
                 convert_stream_closed_error(self, e)
         else:
             try:
-                msg = yield from_frames(frames,
-                                        deserialize=self.deserialize,
-                                        deserializers=deserializers)
+                msg = yield from_frames(
+                    frames, deserialize=self.deserialize, deserializers=deserializers
+                )
             except EOFError:
                 # Frames possibly garbled or truncated by communication error
                 self.abort()
@@ -212,27 +224,29 @@ class TCP(Comm):
             raise gen.Return(msg)
 
     @gen.coroutine
-    def write(self, msg, serializers=None, on_error='message'):
+    def write(self, msg, serializers=None, on_error="message"):
         stream = self.stream
         bytes_since_last_yield = 0
         if stream is None:
             raise CommClosedError
 
-        frames = yield to_frames(msg,
-                                 serializers=serializers,
-                                 on_error=on_error,
-                                 context={'sender': self._local_addr,
-                                          'recipient': self._peer_addr})
+        frames = yield to_frames(
+            msg,
+            serializers=serializers,
+            on_error=on_error,
+            context={"sender": self._local_addr, "recipient": self._peer_addr},
+        )
 
         try:
             lengths = [nbytes(frame) for frame in frames]
-            length_bytes = ([struct.pack('Q', len(frames))] +
-                            [struct.pack('Q', x) for x in lengths])
-            if PY3 and sum(lengths) < 2**17:  # 128kiB
-                b = b''.join(length_bytes + frames)  # small enough, send in one go
+            length_bytes = [struct.pack("Q", len(frames))] + [
+                struct.pack("Q", x) for x in lengths
+            ]
+            if PY3 and sum(lengths) < 2 ** 17:  # 128kiB
+                b = b"".join(length_bytes + frames)  # small enough, send in one go
                 stream.write(b)
             else:
-                stream.write(b''.join(length_bytes))  # avoid large memcpy, send in many
+                stream.write(b"".join(length_bytes))  # avoid large memcpy, send in many
 
                 for frame in frames:
                     # Can't wait for the write() Future as it may be lost
@@ -263,7 +277,7 @@ class TCP(Comm):
             try:
                 # Flush the stream's write buffer by waiting for a last write.
                 if stream.writing():
-                    yield stream.write(b'')
+                    yield stream.write(b"")
                 stream.socket.shutdown(socket.SHUT_RDWR)
             except EnvironmentError:
                 pass
@@ -294,37 +308,42 @@ class TLS(TCP):
         TCP._read_extra(self)
         sock = self.stream.socket
         if sock is not None:
-            self._extra.update(peercert=sock.getpeercert(),
-                               cipher=sock.cipher())
-            cipher, proto, bits = self._extra['cipher']
-            logger.debug("TLS connection with %r: protocol=%s, cipher=%s, bits=%d",
-                         self._peer_addr, proto, cipher, bits)
+            self._extra.update(peercert=sock.getpeercert(), cipher=sock.cipher())
+            cipher, proto, bits = self._extra["cipher"]
+            logger.debug(
+                "TLS connection with %r: protocol=%s, cipher=%s, bits=%d",
+                self._peer_addr,
+                proto,
+                cipher,
+                bits,
+            )
 
 
 def _expect_tls_context(connection_args):
-    ctx = connection_args.get('ssl_context')
+    ctx = connection_args.get("ssl_context")
     if not isinstance(ctx, ssl.SSLContext):
-        raise TypeError("TLS expects a `ssl_context` argument of type "
-                        "ssl.SSLContext (perhaps check your TLS configuration?)"
-                        "  Instead got %s" % str(ctx))
+        raise TypeError(
+            "TLS expects a `ssl_context` argument of type "
+            "ssl.SSLContext (perhaps check your TLS configuration?)"
+            "  Instead got %s" % str(ctx)
+        )
     return ctx
 
 
 class RequireEncryptionMixin(object):
-
     def _check_encryption(self, address, connection_args):
-        if not self.encrypted and connection_args.get('require_encryption'):
+        if not self.encrypted and connection_args.get("require_encryption"):
             # XXX Should we have a dedicated SecurityError class?
-            raise RuntimeError("encryption required by Dask configuration, "
-                               "refusing communication from/to %r"
-                               % (self.prefix + address,))
+            raise RuntimeError(
+                "encryption required by Dask configuration, "
+                "refusing communication from/to %r" % (self.prefix + address,)
+            )
 
 
 class BaseTCPConnector(Connector, RequireEncryptionMixin):
     if PY3:  # see github PR #2403 discussion for more info
-        _executor = ThreadPoolExecutor(2)
-        _resolver = netutil.ExecutorResolver(close_executor=False,
-                                             executor=_executor)
+        _executor = ThreadPoolExecutor(2, thread_name_prefix="TCP-Executor")
+        _resolver = netutil.ExecutorResolver(close_executor=False, executor=_executor)
     else:
         _resolver = None
     client = TCPClient(resolver=_resolver)
@@ -336,9 +355,9 @@ class BaseTCPConnector(Connector, RequireEncryptionMixin):
         kwargs = self._get_connect_args(**connection_args)
 
         try:
-            stream = yield BaseTCPConnector.client.connect(ip, port,
-                                          max_buffer_size=MAX_BUFFER_SIZE,
-                                          **kwargs)
+            stream = yield BaseTCPConnector.client.connect(
+                ip, port, max_buffer_size=MAX_BUFFER_SIZE, **kwargs
+            )
 
             # Under certain circumstances tornado will have a closed connnection with an error and not raise
             # a StreamClosedError.
@@ -352,14 +371,13 @@ class BaseTCPConnector(Connector, RequireEncryptionMixin):
             convert_stream_closed_error(self, e)
 
         local_address = self.prefix + get_stream_address(stream)
-        raise gen.Return(self.comm_class(stream,
-                                         local_address,
-                                         self.prefix + address,
-                                         deserialize))
+        raise gen.Return(
+            self.comm_class(stream, local_address, self.prefix + address, deserialize)
+        )
 
 
 class TCPConnector(BaseTCPConnector):
-    prefix = 'tcp://'
+    prefix = "tcp://"
     comm_class = TCP
     encrypted = False
 
@@ -368,19 +386,19 @@ class TCPConnector(BaseTCPConnector):
 
 
 class TLSConnector(BaseTCPConnector):
-    prefix = 'tls://'
+    prefix = "tls://"
     comm_class = TLS
     encrypted = True
 
     def _get_connect_args(self, **connection_args):
         ctx = _expect_tls_context(connection_args)
-        return {'ssl_options': ctx}
+        return {"ssl_options": ctx}
 
 
 class BaseTCPListener(Listener, RequireEncryptionMixin):
-
-    def __init__(self, address, comm_handler, deserialize=True,
-                 default_port=0, **connection_args):
+    def __init__(
+        self, address, comm_handler, deserialize=True, default_port=0, **connection_args
+    ):
         self._check_encryption(address, connection_args)
         self.ip, self.port = parse_host_port(address, default_port)
         self.comm_handler = comm_handler
@@ -390,18 +408,18 @@ class BaseTCPListener(Listener, RequireEncryptionMixin):
         self.bound_address = None
 
     def start(self):
-        self.tcp_server = TCPServer(max_buffer_size=MAX_BUFFER_SIZE,
-                                    **self.server_args)
+        self.tcp_server = TCPServer(max_buffer_size=MAX_BUFFER_SIZE, **self.server_args)
         self.tcp_server.handle_stream = self._handle_stream
-        backlog = int(dask.config.get('distributed.comm.socket-backlog'))
+        backlog = int(dask.config.get("distributed.comm.socket-backlog"))
         for i in range(5):
             try:
                 # When shuffling data between workers, there can
                 # really be O(cluster size) connection requests
                 # on a single worker socket, make sure the backlog
                 # is large enough not to lose any.
-                sockets = netutil.bind_sockets(self.port, address=self.ip,
-                                               backlog=backlog)
+                sockets = netutil.bind_sockets(
+                    self.port, address=self.ip, backlog=backlog
+                )
             except EnvironmentError as e:
                 # EADDRINUSE can happen sporadically when trying to bind
                 # to an ephemeral port
@@ -430,11 +448,10 @@ class BaseTCPListener(Listener, RequireEncryptionMixin):
         if stream is None:
             # Preparation failed
             return
-        logger.debug("Incoming connection from %r to %r",
-                     address, self.contact_address)
+        logger.debug("Incoming connection from %r to %r", address, self.contact_address)
         local_address = self.prefix + get_stream_address(stream)
         comm = self.comm_class(stream, local_address, address, self.deserialize)
-        self.comm_handler(comm)
+        yield self.comm_handler(comm)
 
     def get_host_port(self):
         """
@@ -465,7 +482,7 @@ class BaseTCPListener(Listener, RequireEncryptionMixin):
 
 
 class TCPListener(BaseTCPListener):
-    prefix = 'tcp://'
+    prefix = "tcp://"
     comm_class = TCP
     encrypted = False
 
@@ -478,13 +495,13 @@ class TCPListener(BaseTCPListener):
 
 
 class TLSListener(BaseTCPListener):
-    prefix = 'tls://'
+    prefix = "tls://"
     comm_class = TLS
     encrypted = True
 
     def _get_server_args(self, **connection_args):
         ctx = _expect_tls_context(connection_args)
-        return {'ssl_options': ctx}
+        return {"ssl_options": ctx}
 
     @gen.coroutine
     def _prepare_stream(self, stream, address):
@@ -492,9 +509,12 @@ class TLSListener(BaseTCPListener):
             yield stream.wait_for_handshake()
         except EnvironmentError as e:
             # The handshake went wrong, log and ignore
-            logger.warning("Listener on %r: TLS handshake failed with remote %r: %s",
-                           self.listen_address, address,
-                           getattr(e, "real_error", None) or e)
+            logger.warning(
+                "Listener on %r: TLS handshake failed with remote %r: %s",
+                self.listen_address,
+                address,
+                getattr(e, "real_error", None) or e,
+            )
         else:
             raise gen.Return(stream)
 
@@ -524,7 +544,7 @@ class BaseTCPBackend(Backend):
     def get_local_address_for(self, loc):
         host, port = parse_host_port(loc)
         host = ensure_ip(host)
-        if ':' in host:
+        if ":" in host:
             local_host = get_ipv6(host)
         else:
             local_host = get_ip(host)
@@ -541,5 +561,5 @@ class TLSBackend(BaseTCPBackend):
     _listener_class = TLSListener
 
 
-backends['tcp'] = TCPBackend()
-backends['tls'] = TLSBackend()
+backends["tcp"] = TCPBackend()
+backends["tls"] = TLSBackend()
