@@ -24,13 +24,13 @@ def itemsize(dt):
 @dask_serialize.register(np.ndarray)
 def serialize_numpy_ndarray(x):
     if x.dtype.hasobject:
-        header = {'pickle': True}
+        header = {"pickle": True}
         frames = [pickle.dumps(x)]
         return header, frames
 
     # We cannot blindly pickle the dtype as some may fail pickling,
     # so we have a mixture of strategies.
-    if x.dtype.kind == 'V':
+    if x.dtype.kind == "V":
         # Preserving all the information works best when pickling
         try:
             # Only use stdlib pickle as cloudpickle is slow when failing
@@ -53,31 +53,29 @@ def serialize_numpy_ndarray(x):
     elif x.flags.c_contiguous or x.flags.f_contiguous:
         # Avoid a copy and respect order when unserializing
         strides = x.strides
-        data = x.ravel(order='K')
+        data = x.ravel(order="K")
     else:
         x = np.ascontiguousarray(x)
         strides = x.strides
         data = x.ravel()
 
     if data.dtype.fields or data.dtype.itemsize > 8:
-        data = data.view('u%d' % gcd(x.dtype.itemsize, 8))
+        data = data.view("u%d" % gcd(x.dtype.itemsize, 8))
 
     try:
         data = data.data
     except ValueError:
         # "ValueError: cannot include dtype 'M' in a buffer"
-        data = data.view('u%d' % gcd(x.dtype.itemsize, 8)).data
+        data = data.view("u%d" % gcd(x.dtype.itemsize, 8)).data
 
-    header = {'dtype': dt,
-              'shape': x.shape,
-              'strides': strides}
+    header = {"dtype": dt, "shape": x.shape, "strides": strides}
 
     if x.nbytes > 1e5:
         frames = frame_split_size([data])
     else:
         frames = [data]
 
-    header['lengths'] = [x.nbytes]
+    header["lengths"] = [x.nbytes]
 
     return header, frames
 
@@ -88,50 +86,68 @@ def deserialize_numpy_ndarray(header, frames):
         if len(frames) > 1:
             frames = merge_frames(header, frames)
 
-        if header.get('pickle'):
+        if header.get("pickle"):
             return pickle.loads(frames[0])
 
-        is_custom, dt = header['dtype']
+        is_custom, dt = header["dtype"]
         if is_custom:
             dt = pickle.loads(dt)
         else:
             dt = np.dtype(dt)
 
-        x = np.ndarray(header['shape'], dtype=dt, buffer=frames[0],
-                       strides=header['strides'])
+        x = np.ndarray(
+            header["shape"], dtype=dt, buffer=frames[0], strides=header["strides"]
+        )
 
         return x
 
 
+@dask_serialize.register(np.ma.core.MaskedConstant)
+def serialize_numpy_ma_masked(x):
+    return {}, []
+
+
+@dask_deserialize.register(np.ma.core.MaskedConstant)
+def deserialize_numpy_ma_masked(header, frames):
+    return np.ma.masked
+
+
 @dask_serialize.register(np.ma.core.MaskedArray)
 def serialize_numpy_maskedarray(x):
-    # Separate elements of the masked array that we need to deal with discretely.
-    data = x.data
-    mask = x.mask
-    fill_value = x.fill_value
+    data_header, frames = serialize_numpy_ndarray(x.data)
+    header = {"data-header": data_header, "nframes": len(frames)}
 
-    # Make use of existing numpy serialization for the two ndarray elements of
-    # the masked array.
-    data_header, data_frames = serialize_numpy_ndarray(data)
-    mask_header, mask_frames = serialize_numpy_ndarray(mask)
+    # Serialize mask if present
+    if x.mask is not np.ma.nomask:
+        mask_header, mask_frames = serialize_numpy_ndarray(x.mask)
+        header["mask-header"] = mask_header
+        frames += mask_frames
 
-    header = {"data-header": data_header,
-              "mask-header": mask_header,
-              "fill_value": fill_value,
-              "nframes": len(data_frames)}
-    return header, data_frames + mask_frames
+    # Only a few dtypes have python equivalents msgpack can serialize
+    if isinstance(x.fill_value, (np.integer, np.floating, np.bool_)):
+        serialized_fill_value = (False, x.fill_value.item())
+    else:
+        serialized_fill_value = (True, pickle.dumps(x.fill_value))
+    header["fill-value"] = serialized_fill_value
+
+    return header, frames
 
 
 @dask_deserialize.register(np.ma.core.MaskedArray)
 def deserialize_numpy_maskedarray(header, frames):
-    data_frames = frames[:header["nframes"]]
-    mask_frames = frames[header["nframes"]:]
     data_header = header["data-header"]
-    mask_header = header["mask-header"]
-
-    # Get the individual elements of the masked array in order to reconstruct.
+    data_frames = frames[: header["nframes"]]
     data = deserialize_numpy_ndarray(data_header, data_frames)
-    mask = deserialize_numpy_ndarray(mask_header, mask_frames)
-    fill_value = header["fill_value"]
+
+    if "mask-header" in header:
+        mask_header = header["mask-header"]
+        mask_frames = frames[header["nframes"] :]
+        mask = deserialize_numpy_ndarray(mask_header, mask_frames)
+    else:
+        mask = np.ma.nomask
+
+    pickled_fv, fill_value = header["fill-value"]
+    if pickled_fv:
+        fill_value = pickle.loads(fill_value)
 
     return np.ma.masked_array(data, mask=mask, fill_value=fill_value)
