@@ -1,4 +1,4 @@
-from collections import MutableMapping
+from distributed.compatibility import MutableMapping
 from distributed.utils import log_errors, tokey
 
 
@@ -15,26 +15,30 @@ class PublishExtension(object):
         self.scheduler = scheduler
         self.datasets = dict()
 
-        handlers = {'publish_list': self.list,
-                    'publish_put': self.put,
-                    'publish_get': self.get,
-                    'publish_delete': self.delete}
+        handlers = {
+            "publish_list": self.list,
+            "publish_put": self.put,
+            "publish_get": self.get,
+            "publish_delete": self.delete,
+        }
 
         self.scheduler.handlers.update(handlers)
-        self.scheduler.extensions['publish'] = self
+        self.scheduler.extensions["publish"] = self
 
     def put(self, stream=None, keys=None, data=None, name=None, client=None):
         with log_errors():
             if name in self.datasets:
                 raise KeyError("Dataset %s already exists" % name)
-            self.scheduler.client_desires_keys(keys, 'published-%s' % tokey(name))
-            self.datasets[name] = {'data': data, 'keys': keys}
-            return {'status':  'OK', 'name': name}
+            self.scheduler.client_desires_keys(keys, "published-%s" % tokey(name))
+            self.datasets[name] = {"data": data, "keys": keys}
+            return {"status": "OK", "name": name}
 
     def delete(self, stream=None, name=None):
         with log_errors():
-            out = self.datasets.pop(name, {'keys': []})
-            self.scheduler.client_releases_keys(out['keys'], 'published-%s' % tokey(name))
+            out = self.datasets.pop(name, {"keys": []})
+            self.scheduler.client_releases_keys(
+                out["keys"], "published-%s" % tokey(name)
+            )
 
     def list(self, *args):
         with log_errors():
@@ -53,6 +57,7 @@ class Datasets(MutableMapping):
     client : distributed.client.Client
 
     """
+
     def __init__(self, client):
         self.__client = client
 
